#include <Wire.h>
#include <LiquidCrystal_I2C.h>
#include "MAX30105.h"
#include "spo2_algorithm.h"
#include "heartRate.h"

MAX30105 particleSensor;

#define MAX_BRIGHTNESS 255

#if defined(__AVR_ATmega328P__) || defined(__AVR_ATmega168__)

uint16_t irBuffer[100]; //infrared LED sensor data
uint16_t redBuffer[100]; //red LED sensor data
#else
uint32_t irBuffer[100]; //infrared LED sensor data
uint32_t redBuffer[100]; //red LED sensor data
#endif

int32_t bufferLength; //data length
int32_t spo2; //SPO2 value
int8_t validSPO2; //indicator to show if the SPO2 calculation is valid
int32_t heartRate; //heart rate value
int8_t validHeartRate; //indicator to show if the heart rate calculation is valid

byte pulseLED = 11; //Must be on PWM pin
byte readLED = 13; //Blinks with each data read

LiquidCrystal_I2C lcd(0x27, 16, 2); 

void setup()
{
  lcd.begin(16, 2);
  lcd.setBacklight(MAX_BRIGHTNESS);

  pinMode(pulseLED, OUTPUT);
  pinMode(readLED, OUTPUT);

  // Initialize sensor
  if (!particleSensor.begin(Wire, I2C_SPEED_FAST))
  {
    lcd.clear();
    lcd.print("Sensor not found!");
    while (1);
  }

  byte ledBrightness = 60;
  byte sampleAverage = 4; 
  byte ledMode = 2; 
  byte sampleRate = 100;
  int pulseWidth = 411;
  int adcRange = 4096; 

  particleSensor.setup(ledBrightness, sampleAverage, ledMode, sampleRate, pulseWidth, adcRange);

  loop();
}

void loop()
{
  bufferLength = 100; //buffer length of 100 stores 4 seconds of samples running at 25sps

  for (byte i = 0; i < bufferLength; i++)
  {
    while (particleSensor.available() == false) //do we have new data?
      particleSensor.check(); //Check the sensor for new data

    redBuffer[i] = particleSensor.getRed();
    irBuffer[i] = particleSensor.getIR();
    particleSensor.nextSample();
  }

  //After gathering 100 new samples, calculate heart rate and SpO2
  maxim_heart_rate_and_oxygen_saturation(irBuffer, bufferLength, redBuffer, &spo2, &validSPO2, &heartRate, &validHeartRate);

  if (validSPO2 == 0 && spo2 > 0)
  {
    validSPO2 = 1;
  }

  //Print the values to the LCD
  lcd.clear();
  lcd.setCursor(0, 0);
  if (validSPO2)
  {
    lcd.print(F("SpO2: "));
    lcd.print(spo2);
    lcd.print(F("%"));
  }
  else
  {
    lcd.print(F("SpO2: --"));
  }

  lcd.setCursor(0, 1);
  if (validHeartRate)
  {
    lcd.print(F("HR: "));
    lcd.print(heartRate);
    lcd.print(F(" bpm"));
  }
  else
  {
    lcd.print(F("HR: ---"));
  }

  digitalWrite(readLED, !digitalRead(readLED));

  delay(500); // Delay for 1 second before taking the next reading
}
